
// Name
//===================================
//
// BlockSystem
//


#ifndef BLOCK_SYSTEM_H
#define BLOCK_SYSTEM_H


// FORWARD DECLARATIONS
//===================================

    class BlockType;
    class Camera;
    class EventSystem;
    class GameState;
    class Inventory;
    class Level;
    class Sprite;


// DEPENDENCIES
//===================================

    #include "block.h"
    #include "unit.h"
    #include <vector>


// CLASS
//===================================

    class BlockSystem
    {
        public:
            BlockSystem();
            ~BlockSystem();

            void update( Camera& camera, Input& input );
            void render( Graphics& graphics, Camera& camera, bool priority = false );
            void addBlock( int x, int y, BlockType* type, int location );
            void clearBlocks();
            void collide( Sprite& sprite, Level& level, EventSystem& events, Inventory& inventory, Camera& camera, GameState* state, Graphics& graphics );
            void blocksFromMap( Map& lvmap, Camera& camera );

        private:
            static const int CAMERA_PADDING = 2;
            static const int LIMIT = ( Unit::WINDOW_WIDTH_BLOCKS + ( CAMERA_PADDING * 2 ) ) * ( Unit::WINDOW_HEIGHT_BLOCKS + ( CAMERA_PADDING * 2 ) );
            std::vector<BlockType*> block_types_;
            BlockType* blockType( int n ) const;

            int blocks_used_ = 0;
            Block blocks_[ LIMIT ];

            void clearBlockTypes();
    };

#endif // BLOCK_SYSTEM_H
