

// Name
//===================================
//
// Palette
//


#ifndef PALETTE_H
#define PALETTE_H


// FORWARD DECLARATIONS
//===================================


// DEPENDENCIES
//===================================

    #include <SDL2/SDL_image.h>


// CLASS
//===================================

    class Palette
    {
        public:
            static const Uint8 DEFAULT_VALUE = 255;

            // Don't apply specific #s.
            enum PaletteType
            {
                GRAYSCALE,
                CLASSIC_GREEN,
                MIDNIGHT_BLUE,
                __NULL // Must be last.
            };

            struct PaletteSet
            {
                PaletteType type;
                int bg;
            };

            Palette( PaletteType p = PaletteType::GRAYSCALE, unsigned int bg = 1 );
            Palette( PaletteSet p );
            ~Palette();

            void changePalette( PaletteType p, unsigned int bg = 1 );
            void changePalette( PaletteSet p );
            void applyPalette( SDL_Surface* s ) const;
            SDL_Color& getColor( unsigned int n = 1 );
            SDL_Color& bg();
            Uint8 bgR() const;
            Uint8 bgG() const;
            Uint8 bgB() const;
            Uint8 bgA() const;
            static PaletteSet defaultPaletteSet();

        private:
            static const int COLOR_LIMIT = 7;
            static const int PALETTE_LIMIT = 3;
            const SDL_Color PALETTE_SETS[ PALETTE_LIMIT ][ COLOR_LIMIT ] =
            {
                {
                    { 255, 255, 255, 0   },
                    { 255, 255, 255, 255 },
                    { 161, 161, 161, 255 },
                    { 121, 121, 121, 255 },
                    {  81,  81,  81, 255 },
                    {  40,  40,  40, 255 },
                    {   0,   0,   0, 255 }
                },
                {
                    { 255, 255, 255, 0   },
                    { 255, 255, 255, 255 },
                    { 190, 241,  51, 255 },
                    { 139, 172,  15, 255 },
                    {  48,  98,  48, 255 },
                    {  15,  56,  15, 255 },
                    {   0,   0,   0, 255 }
                },
                {
                    { 255, 255, 255, 0   },
                    { 255, 255, 255, 255 },
                    { 216, 184, 248, 255 },
                    { 152, 120, 248, 255 },
                    { 104,  68, 252, 255 },
                    {  68,  40, 188, 255 },
                    {   0,   0,   0, 255 }
                }
            };

            SDL_Color colors_[ COLOR_LIMIT ];
            unsigned int bg_ = 1;

            void createPalette( PaletteType p );
            unsigned int testColor( unsigned int n ) const;
            Palette::PaletteType testPal( Palette::PaletteType p ) const;
    };

#endif // PALETTE_H


