




// Name
//===================================
//
// Text
//


// DEPENDENCIES
//===================================

    #include "graphics.h"
    #include "text.h"
    #include "unit.h"


// STATIC PROPERTIES
//===================================

    std::map<char, int> Text::char_conversion_ =
    {
        { '0', 0 },
        { '1', 1 },
        { '2', 2 },
        { '3', 3 },
        { '4', 4 },
        { '5', 5 },
        { '6', 6 },
        { '7', 7 },
        { '8', 8 },
        { '9', 9 },
        { 'a', 10 },
        { 'A', 10 },
        { 'b', 11 },
        { 'B', 11 },
        { 'c', 12 },
        { 'C', 12 },
        { 'd', 13 },
        { 'D', 13 },
        { 'e', 14 },
        { 'E', 14 },
        { 'f', 15 },
        { 'F', 15 },
        { 'g', 16 },
        { 'G', 16 },
        { 'h', 17 },
        { 'H', 17 },
        { 'i', 18 },
        { 'I', 18 },
        { 'j', 19 },
        { 'J', 19 },
        { 'k', 20 },
        { 'K', 20 },
        { 'l', 21 },
        { 'L', 21 },
        { 'm', 22 },
        { 'M', 22 },
        { 'n', 23 },
        { 'N', 23 },
        { 'o', 24 },
        { 'O', 24 },
        { 'p', 25 },
        { 'P', 25 },
        { 'q', 26 },
        { 'Q', 26 },
        { 'r', 27 },
        { 'R', 27 },
        { 's', 28 },
        { 'S', 28 },
        { 't', 29 },
        { 'T', 29 },
        { 'u', 30 },
        { 'U', 30 },
        { 'v', 31 },
        { 'V', 31 },
        { 'w', 32 },
        { 'W', 32 },
        { 'x', 33 },
        { 'X', 33 },
        { 'y', 34 },
        { 'Y', 34 },
        { 'z', 35 },
        { 'Z', 35 },
        { '?', 36 },
        { '!', 37 },
        { '¡', 38 },
        { '¿', 39 },
        { '@', 40 },
        { '#', 41 },
        { '$', 42 },
        { '%', 43 },
        { '^', 44 },
        { '&', 45 },
        { '*', 46 },
        { '(', 47 },
        { ')', 48 },
        { '[', 49 },
        { ']', 50 },
        { ',', 51 },
        { '.', 52 },
        { '/', 53 },
        { '\'', 54 },
        { ':', 55 },
        { ';', 56 },
        { '{', 57 },
        { '}', 58 },
        { '-', 59 },
        { '_', 60 },
        { '=', 61 },
        { '"', 62 },
        { '/', 63 },
        { '\\', 64 },
        { '~', 65 },
        { '|', 66 },
        { '+', 67 },
        { 'í', 74 },
        { PTS, 68 },
        { HEART, 69 },
        { RIGHT_ARROW, 70 },
        { LEFT_ARROW, 71 },
        { DOWN_ARROW, 72 },
        { UP_ARROW, 73 },
        { ' ', 127 }
    };


// METHODS
//===================================

    Text::Text
    (
        std::string words,
        int x,
        int y,
        FontShade shade,
        FontAlign align,
        bool blink
    )
    :
        words_ ( words ),
        x_ ( x ),
        y_ ( y ),
        rect_ ( { x, y, 8, 8 } ),
        shade_ ( shade ),
        align_ ( align ),
        blink_ ( blink )
    {};

    Text::~Text() {};

    void Text::render( Graphics& graphics, Camera* camera, FontShade shade )
    {
        int x_mem = rect_.x;
        int frame = 0;

        if ( align_ == FA_CENTER )
            rect_.x = centerX();

        for ( int i = 0; i < words_.length(); ++i )
        {
            // For some reason, extended ASCII that are numerically below 0 are 2 chars, the 1st coming out as a tacked-on 0 beforehand..
            // Skipping to the 2nd half o' the ASCII fixes this.
            if ( words_[ i ] < 0 )
                ++i;

            frame = char_conversion_[ words_[ i ] ];
            rect_.x += 8;
            graphics.renderObject( Graphics::SpriteSheet::CHARSET, { frameX( frame ), frameY( frame, shade ), 8, 8 }, rect_ );
        }

        rect_.x = x_mem;
    };

    int Text::centerX() const
    {
        return Unit::MiniBlocksToPixels( floor ( Unit::WINDOW_WIDTH_MINIBLOCKS / 2 ) - floor( words_.length() / 2 ) );
    };

    int Text::shadeOffset( FontShade shade ) const
    {
        if ( shade == FontShade::__NULL )
        {
            return shadeNum() * 32;
        }
        else
        {
            return (int)shade * 32;
        }
    };

    int Text::shadeNum() const
    {
        if ( shade_ == FontShade::__NULL )
        {
            return 0;
        }
        else
        {
            return (int)shade_;
        }
    };

    int Text::frameX( int n ) const
    {
        return Unit::MiniBlocksToPixels( n % CHARSET_WIDTH_MINI_BLOCKS );
    };

    int Text::frameY( int n, FontShade shade ) const
    {
        return shadeOffset( shade ) + Unit::MiniBlocksToPixels( floor( n / CHARSET_WIDTH_MINI_BLOCKS ) );
    };

    void Text::renderNumber( Graphics& graphics, int n, int digits, Camera* camera, FontShade shade )
    {
        int nDigits = numODigits( n );

        if ( digits < 1 )
        {
            digits = nDigits;
        }

        for ( int i = digits; i > 0; --i )
        {
            int d = 0;

            if ( i <= nDigits )
            {
                d = getDigit( n, i );
            }

            graphics.renderObject( Graphics::SpriteSheet::CHARSET, {8 * d, shadeOffset( shade ) ,8,8}, rect_ );
            rect_.x += 8;
        }
        rect_.x = x_;
    };

    int Text::getDigit( int n, int d, int remain ) const
    {
        return int( floor( n / pow( 10, d-1 ) ) ) % remain;
    };

    int Text::numODigits( int n ) const
    {
        if ( n == 0 )
        {
            return 1;
        }
        else
        {
            for ( int i = 0; i < 99; ++i )
            {
                if ( abs( n ) < pow( 10, i ) )
                {
                    return i;
                }
            }
        }

        return -1;
    };

    void Text::update()
    {
        if ( blink_ )
        {
            if ( blink_timer_.hit() )
            {
                if ( blink_dir_ == Direction::Vertical::DOWN )
                {
                    --blink_counter_;

                    if ( blink_counter_.value() == 0 )
                        blink_dir_ = Direction::Vertical::UP;
                }
                else
                {
                    ++blink_counter_;

                    if ( blink_counter_.value() == NUM_OF_FONT_SHADES-1 )
                        blink_dir_ = Direction::Vertical::DOWN;
                }
            }

            blink_timer_.update();

            if ( blink_counter_.value() >= 0 && blink_counter_.value() < NUM_OF_FONT_SHADES )
                shade_ = (FontShade)blink_counter_.value();
        }
    };
